import sys

sys.path.append("../../python/")
sys.path.append("../../../lib/python/")
sys.path.append("../../../lib/python/Debug/")

import os
import unittest
import idyntree.swig as iDynTree;
import numpy as np
import random

class ExtWrenchesAndJointTorquesEstimatorUnitTest(unittest.TestCase):
    def tol(self):
        return 1e-10;

    def places(self):
        return 7;

    def checkApproxEqual(self,val1,val2,msg):
        self.assertAlmostEqual(val1,val2,self.places(),msg);


    def checkVecApproxEqual(self,p1,p2,msg):
        msgMore = "val1 = " + str(p1) + " and val2 = " + str(p2)
        for i in range(0,len(p1)):
            self.checkApproxEqual(p1[i],p2[i],msg+":"+msgMore);

    def testcomputeSubModelMatrixRelatingFTSensorsMeasuresAndKinematics(self):

        # Define considered joints
        consideredJoints = iDynTree.StringVector()
        consideredJoints.push_back("torso_pitch")
        consideredJoints.push_back("torso_roll")
        consideredJoints.push_back("torso_yaw")
        consideredJoints.push_back("neck_pitch")
        consideredJoints.push_back("neck_roll")
        consideredJoints.push_back("neck_yaw")
        consideredJoints.push_back("l_shoulder_pitch")
        consideredJoints.push_back("l_shoulder_roll")
        consideredJoints.push_back("l_shoulder_yaw")
        consideredJoints.push_back("l_elbow")
        consideredJoints.push_back("r_shoulder_pitch")
        consideredJoints.push_back("r_shoulder_roll")
        consideredJoints.push_back("r_shoulder_yaw")
        consideredJoints.push_back("r_elbow")
        consideredJoints.push_back("l_hip_pitch")
        consideredJoints.push_back("l_hip_roll")
        consideredJoints.push_back("l_hip_yaw")
        consideredJoints.push_back("l_knee")
        consideredJoints.push_back("l_ankle_pitch")
        consideredJoints.push_back("l_ankle_roll")
        consideredJoints.push_back("r_hip_pitch")
        consideredJoints.push_back("r_hip_roll")
        consideredJoints.push_back("r_hip_yaw")
        consideredJoints.push_back("r_knee")
        consideredJoints.push_back("r_ankle_pitch")
        consideredJoints.push_back("r_ankle_roll")

        # Load model
        estimator = iDynTree.ExtWrenchesAndJointTorquesEstimator()
        testModel = os.path.join(os.path.dirname(__file__), "../../../../src/tests/data/iCubDarmstadt01.urdf")
        ok = estimator.loadModelAndSensorsFromFileWithSpecifiedDOFs(testModel, consideredJoints)
        self.assertTrue(ok)

        # Set random state
        dofs = estimator.model().getNrOfDOFs()
        q = iDynTree.JointPosDoubleArray.FromPython([random.random() for i in range(0, dofs)])
        dq = iDynTree.JointDOFsDoubleArray.FromPython([random.random() for i in range(0, dofs)])
        ddq = iDynTree.JointDOFsDoubleArray.FromPython([random.random() for i in range(0, dofs)])
        grav = iDynTree.Vector3.FromPython([0.0, 0.0, -9.81])

        root_link_index = estimator.model().getFrameIndex("root_link")
        estimator.updateKinematicsFromFixedBase(q,dq,ddq,root_link_index,grav)

        # Set unknown contact forces
        fullBodyUnknowns = iDynTree.LinkUnknownWrenchContacts(estimator.model())
        unknown = iDynTree.UnknownWrenchContact()
        unknown.unknownType = iDynTree.FULL_WRENCH
        unknown.contactPoint.zero()
        fullBodyUnknowns.addNewContactInFrame(estimator.model(), root_link_index, unknown)

        # Compute expected FT measures
        # First of all, we build the 6*nrOfFTsensors vector composed by the known FT sensors measures
        sensOffset = iDynTree.SensorsMeasurements(estimator.model().sensors())
        estimatedContactWrenches = iDynTree.LinkContactWrenches(estimator.model())
        estimatedJointTorques = iDynTree.JointDOFsDoubleArray(estimator.model())

        ok = estimator.computeExpectedFTSensorsMeasurements(fullBodyUnknowns, sensOffset, estimatedContactWrenches, estimatedJointTorques)
        self.assertTrue(ok)

        nrOfFTSensors = estimator.model().sensors().getNrOfSensors(iDynTree.SIX_AXIS_FORCE_TORQUE)

        w = np.zeros(6*nrOfFTSensors)
        for ft in range(0, nrOfFTSensors):
            ft_meas = iDynTree.Wrench()
            sensOffset.getMeasurement(iDynTree.SIX_AXIS_FORCE_TORQUE, ft, ft_meas)
            w[range(6*ft, 6*ft+6)] = ft_meas.toNumPy()

        A = iDynTree.MatrixDynSizeVector()
        b = iDynTree.VectorDynSizeVector()
        subModelIDs = iDynTree.IndexVector()
        baseLinkIndeces = iDynTree.IndexVector()


        # Test computeSubModelMatrixRelatingFTSensorsMeasuresAndKinematics
        ok = estimator.computeSubModelMatrixRelatingFTSensorsMeasuresAndKinematics(fullBodyUnknowns, A, b, subModelIDs, baseLinkIndeces)
        self.assertTrue(ok)
        self.assertEqual(A.size(), b.size())
        self.assertEqual(A.size(), subModelIDs.size())
        self.assertEqual(A.size(), baseLinkIndeces.size())

        for i in range(0, A.size()):
            bNumPy = b[i].toNumPy()
            bCheck = A[i].toNumPy()@w
            self.checkVecApproxEqual(bNumPy, bCheck, "computeSubModelMatrixRelatingFTSensorsMeasuresAndKinematics check failed")


if __name__ == '__main__':
    # initalize the seed to have predictable results
    random.seed(0)
    unittest.main()



#!/usr/bin/env python3

# SPDX-FileCopyrightText: Fondazione Istituto Italiano di Tecnologia (IIT)
# SPDX-License-Identifier: BSD-3-Clause

import argparse
import os


def generate_documentation_files(tag: str, input_files_path: str, output_files_path: str,
                                 src_folder: str, main_page: str, additional_pages=[]):
    src_path = os.path.normpath(src_folder)
    input_files_path = os.path.normpath(input_files_path)
    output_files_path = os.path.normpath(output_files_path)
    main_page = os.path.normpath(main_page)
    _, main_page_extension = os.path.splitext(main_page)

    main_page_md = ''
    if main_page_extension == '.md':
        main_page_md = main_page

    folders = [os.path.normpath(folder[0]) for folder in os.walk(src_path)
               if os.path.basename(os.path.normpath(folder[0])) == 'include']

    output_doxyfile_filename = 'Doxyfile-' + tag
    output_doxyfile_mcss_filename = 'Doxyfile-mcss-' + tag
    output_conf_filename = 'conf-' + tag + '.py'

    BLF_VERSION_PATH = '.'
    if tag != 'master':
        BLF_VERSION_PATH = tag

    BLF_DOCUMENTATION_ROOT = '.'
    if tag != 'master':
        BLF_DOCUMENTATION_ROOT = '..'

    try:
        input_file = open(os.path.join(input_files_path, 'Doxyfile.in'), 'rt')
        output_file = open(os.path.join(input_files_path, output_doxyfile_filename), 'wt')

        for line in input_file:
            output_file.write(line.replace('@BLF_INCLUDE_FOLDERS@',
                                           "\"" + "\" \"".join(folders) + "\"")
                              .replace('@BLF_MAIN_PAGE@', main_page)
                              .replace('@BLF_ADDITIONAL_FILES@', "\"" + "\" \"".join(additional_pages) + "\""))

    # Do something with the file
    except FileNotFoundError:
        print("Invalid doxyfile input file.")
    finally:
        input_file.close()
        output_file.close()

    try:
        input_file = open(os.path.join(input_files_path, 'Doxyfile-mcss.in'), 'rt')
        output_file = open(os.path.join(input_files_path, output_doxyfile_mcss_filename), 'wt')

        for line in input_file:
            output_file.write(line.replace('@BLF_TAG@', tag).replace('@BLF_VERSION_PATH@', BLF_VERSION_PATH).replace(
                '@BLF_MAIN_PAGE@', main_page_md))

        # Do something with the file
    except FileNotFoundError:
        print("Invalid doxyfile-mcss input file.")
    finally:
        input_file.close()
        output_file.close()

    try:
        input_file = open(os.path.join(input_files_path, 'conf.py.in'), 'rt')
        output_file = open(os.path.join(input_files_path, output_conf_filename), 'wt')

        for line in input_file:
            output_file.write(
                line.replace('@BLF_TAG@', tag).replace('@BLF_DOCUMENTATION_ROOT@', BLF_DOCUMENTATION_ROOT))

        # Do something with the file
    except FileNotFoundError:
        print("Invalid conf.py input file.")
    finally:
        input_file.close()
        output_file.close()


if __name__ == '__main__':
    parser = argparse.ArgumentParser(
        description='generate_doxyfile.py is a python script useful to generate the Doxyfile from Doxyfile.in.')
    parser.add_argument('--tag', type=str, default='master', required=False, help='Version of project.')
    parser.add_argument('--input_files_path', type=str, default='./', required=False, help='Path of the input files.')
    parser.add_argument('--output_files_path', type=str, default='./', required=False, help='Path of the output files.')
    parser.add_argument('--src_folder', type=str, required=True, help='Path of the src folder.')
    parser.add_argument('--main_page', type=str, required=True, help='Path to the the main page.')
    args = parser.parse_args()

    generate_documentation_files(tag=args.tag,
                                 input_files_path=args.input_files_path,
                                 output_files_path=args.output_files_path,
                                 src_folder=args.src_folder,
                                 main_page=args.main_page)



#!/usr/bin/env python3

# SPDX-FileCopyrightText: Fondazione Istituto Italiano di Tecnologia (IIT)
# SPDX-License-Identifier: BSD-3-Clause

import argparse

from idyntree.visualize import MeshcatVisualizer


def main():
    parser = argparse.ArgumentParser(
        description="Display the model in the meshcat visualizer."
    )
    parser.add_argument("--model", "-m", type=str, required=True, help="Model path.")
    parser.add_argument("--alpha", "-a", type=float, required=False, help="Alpha channel of the model.")

    args = parser.parse_args()

    # Load the visualizer
    viz = MeshcatVisualizer()
    viz.load_model_from_file(args.model, color=args.alpha)
    viz.open()

    run = True
    while run:
        quit = input("Enter q or Q to Quit: ")
        run = (quit != "q") and (quit != "Q")


if __name__ == "__main__":
    main()



"""Tests for idyntree-high-level Python bindings."""
import itertools
import math
import unittest

import idyntree.pybind as iDynTree
import numpy as np


class IDynTreeHighLevelTest(unittest.TestCase):

  def _transform_equality_func(self,
                               expected: iDynTree.Transform,
                               other: iDynTree.Transform,
                               msg: str = None):
    equals = True
    error_msg = [msg] if msg else [""]
    for r, c in itertools.product(range(3), range(3)):
      check = math.isclose(expected.rotation[r, c], other.rotation[r, c])
      if not check:
        equals = False
        error_msg.append(
            f"Rotation [{r},{c}]: expected {expected.rotation[r, c]} "
            f"actual {other.rotation[r, c]}")
    for i in range(3):
      check = math.isclose(expected.position[i], other.position[i])
      if not check:
        equals = False
        error_msg.append(f"Position [{i}]: expected {expected.position[i]} "
                         f"actual {other.position[i]}")
    if not equals:
      raise self.failureException("\n".join(error_msg))

  def setUp(self):
    super().setUp()
    self.addTypeEqualityFunc(iDynTree.Transform, self._transform_equality_func)

    model = iDynTree.Model()
    # 3 links.
    link1 = iDynTree.Link()
    link2 = iDynTree.Link()
    link3 = iDynTree.Link()
    link1_idx = model.add_link("link1", link1)
    link2_idx = model.add_link("link2", link2)
    link3_idx = model.add_link("link3", link3)
    self.assertGreaterEqual(link1_idx, 0)
    self.assertGreaterEqual(link2_idx, 0)
    self.assertGreaterEqual(link3_idx, 0)

    # 2 joints. Axis is always Z.
    direction = iDynTree.Direction(0, 0, 1)
    axis = iDynTree.Axis(direction, iDynTree.Position.Zero())

    joint = iDynTree.RevoluteJoint()
    joint.set_attached_links(link1_idx, link2_idx)
    position = iDynTree.Position(0, 1, 0)
    rotation = iDynTree.Rotation(0, 0, 1, 1, 0, 0, 0, 1, 0)
    self._joint1_transform = iDynTree.Transform(rotation, position)
    joint.set_rest_transform(self._joint1_transform)
    joint.set_axis(axis, link2_idx, link1_idx)
    self._joint1_idx = model.add_joint("joint1", joint)
    self.assertGreaterEqual(self._joint1_idx, 0)

    joint = iDynTree.RevoluteJoint()
    joint.set_attached_links(link2_idx, link3_idx)
    position = iDynTree.Position(0, 1, 0)
    rotation = iDynTree.Rotation(0, -1, 0, 1, 0, 0, 0, 0, 1)
    self._joint2_transform = iDynTree.Transform(rotation, position)
    joint.set_rest_transform(self._joint2_transform)
    joint.set_axis(axis, link3_idx, link2_idx)
    self._joint2_idx = model.add_joint("joint2", joint)
    self.assertGreaterEqual(self._joint2_idx, 0)

    # 2 frames.
    position = iDynTree.Position(1, 0.2, -1.5)
    rotation = iDynTree.Rotation(0, 0, 1, 1, 0, 0, 0, 1, 0)
    self._base_frame_transform = iDynTree.Transform(rotation, position)
    model.add_additional_frame_to_link("link1", "base_frame",
                                       self._base_frame_transform)

    position = iDynTree.Position(-0.1, -0.5, 1.1)
    rotation = iDynTree.Rotation(1, 0, 0, 0, -1, 0, 0, 0, -1)
    self._ee_frame_transform = iDynTree.Transform(rotation, position)
    model.add_additional_frame_to_link("link3", "ee_frame",
                                       self._ee_frame_transform)

    self._kin_dyn = iDynTree.KinDynComputations()
    self.assertTrue(self._kin_dyn.load_robot_model(model))

  def test_model_info(self):
    self.assertEqual(2, self._kin_dyn.get_nr_of_degrees_of_freedom())
    self.assertEqual(3, self._kin_dyn.get_nr_of_links())
    # 3 links + 2 explicit frames.
    self.assertEqual(5, self._kin_dyn.get_nr_of_frames())

  def test_floating_base(self):
    # The default should be the first link as there are no reason to change it
    # while generating the traversal.
    self.assertEqual("link1", self._kin_dyn.get_floating_base())

    # Now set a new floating base.
    self.assertTrue(self._kin_dyn.set_floating_base("link3"))
    self.assertEqual("link3", self._kin_dyn.get_floating_base())

  def test_relative_transform(self):
    positions = iDynTree.VectorDynSize(
        self._kin_dyn.get_nr_of_degrees_of_freedom())
    positions.set_zero()

    velocities = iDynTree.VectorDynSize(
        self._kin_dyn.get_nr_of_degrees_of_freedom())
    velocities.set_zero()
    gravity = iDynTree.Vector3()
    gravity.set_zero()
    gravity[2] = -9.81

    # Set a robot state. Use a fixed base state.
    # Set an arbitrary configuration for joint2
    positions[self._joint2_idx] = np.deg2rad(45)
    self._kin_dyn.set_fixed_base_robot_state(positions, velocities, gravity)
    transform = self._kin_dyn.get_relative_transform_for_frames_named(
        "ee_frame", "link2")
    # Construct the expected transform.
    # ee_frame_H_link2 = (link2_H_link3^- * H_joint2 * link3_H_ee_frame) ^-1.
    joint2_var_transform = iDynTree.Transform(
        iDynTree.Rotation.RotZ(positions[self._joint2_idx]),
        iDynTree.Position.Zero())
    expected_transform = (self._joint2_transform * joint2_var_transform *
                          self._ee_frame_transform).inverse()

    self.assertEqual(expected_transform, transform)

  def test_absolute_transform(self):
    positions = iDynTree.VectorDynSize(
        self._kin_dyn.get_nr_of_degrees_of_freedom())
    positions.set_zero()

    velocities = iDynTree.VectorDynSize(
        self._kin_dyn.get_nr_of_degrees_of_freedom())
    velocities.set_zero()
    gravity = iDynTree.Vector3()
    gravity.set_zero()
    gravity[2] = -9.81

    # Arbitrary floating base world transform.
    floating_base_transform = iDynTree.Transform(
        iDynTree.Rotation(0.3224500, -0.9092497, 0.2632318, -0.9453749,
                          -0.3234002, 0.0409703, 0.0478770, -0.2620636,
                          -0.9638622), iDynTree.Position(1, 2, 3))

    base_twist = iDynTree.Twist([0, 0, 0], [0, 0, 0])

    # Set a robot state.
    # Set an arbitrary configuration for joint2
    positions[self._joint2_idx] = np.deg2rad(45)
    self._kin_dyn.set_robot_state(floating_base_transform, positions,
                                  base_twist, velocities, gravity)
    transform = self._kin_dyn.get_world_transform_for_frame_named("ee_frame")
    # Construct the expected transform.
    # world_H_ee_frame = (floating_base_transform * link1_H_link2 *
    #                     link2_H_link3^- * H_joint2 * link3_H_ee_frame).
    joint2_var_transform = iDynTree.Transform(
        iDynTree.Rotation.RotZ(positions[self._joint2_idx]),
        iDynTree.Position.Zero())
    expected_transform = (
        floating_base_transform * self._joint1_transform *
        self._joint2_transform * joint2_var_transform *
        self._ee_frame_transform)

    self.assertEqual(expected_transform, transform)


if __name__ == "__main__":
  unittest.main()



"""Tests for sensors-related Python bindings."""
import itertools
import unittest

import idyntree.pybind as iDynTree


class IDynTreeSensorsTest(unittest.TestCase):

  def test_six_axis_ft_sensor(self):
    sensor = iDynTree.SixAxisForceTorqueSensor()
    self.assertEqual(sensor.sensor_type, iDynTree.SIX_AXIS_FORCE_TORQUE)

  def test_accelerometer_sensor(self):
    sensor = iDynTree.AccelerometerSensor()
    self.assertEqual(sensor.sensor_type, iDynTree.ACCELEROMETER)

  def test_joint_sensor_name(self):
    sensor = iDynTree.SixAxisForceTorqueSensor()
    sensor.name = "a_sensor"
    self.assertEqual(sensor.name, "a_sensor")

  def test_joint_sensor_joint(self):
    sensor = iDynTree.SixAxisForceTorqueSensor()
    sensor.parent_joint = "a_joint"
    self.assertEqual(sensor.parent_joint, "a_joint")

  def test_joint_sensor_joint_index(self):
    sensor = iDynTree.SixAxisForceTorqueSensor()
    sensor.parent_joint_index = 1
    self.assertEqual(sensor.parent_joint_index, 1)

  def test_link_sensor_name(self):
    sensor = iDynTree.AccelerometerSensor()
    sensor.name = "a_sensor"
    self.assertEqual(sensor.name, "a_sensor")

  def test_link_sensor_link(self):
    sensor = iDynTree.AccelerometerSensor()
    sensor.parent_link = "a_link"
    self.assertEqual(sensor.parent_link, "a_link")

  def test_link_sensor_link_index(self):
    sensor = iDynTree.AccelerometerSensor()
    sensor.parent_link_index = 1
    self.assertEqual(sensor.parent_link_index, 1)

  def test_link_sensor_transform(self):
    sensor = iDynTree.AccelerometerSensor()
    position = iDynTree.Position(1, 2, 3)
    rotation = iDynTree.Rotation(0, 0, 1,
                                 1, 0, 0,
                                 0, 1, 0)
    sensor.link_sensor_transform = iDynTree.Transform(rotation, position)

    # Assert the content is the same.
    for r, c in itertools.product(range(3), range(3)):
      self.assertEqual(
          sensor.link_sensor_transform.rotation[r, c], rotation[r, c])
    for i in range(3):
      self.assertEqual(sensor.link_sensor_transform.position[i], position[i])

  def test_sensors_list_creation(self):
    self.assertIsNotNone(iDynTree.SensorsList())

  def test_sensors_list_add_sensors(self):
    sensors = iDynTree.SensorsList()
    sensor = iDynTree.AccelerometerSensor()
    sensor.name = "a_name"
    sensor.parent_link_index = 1
    sensors.add_sensor(sensor)
    self.assertEqual(sensors.get_nr_of_sensors(iDynTree.ACCELEROMETER), 1)
    self.assertEqual(sensors.get_nr_of_sensors(iDynTree.GYROSCOPE), 0)


if __name__ == "__main__":
  unittest.main()



#
# simple Tokenizer for Matlab / Octave code
#

import re

class Token(object):
    NAME = 1
    OP = 2
    COMMENT = 3
    NL = 4
    WHITESPACE = 5

    def __init__(self, content, type):
        self.content = content
        self.type = type

    def __str__(self):
        return '"%s" [%s]' % (self.content, self.type)

    def __repr__(self):
        return self.__str__()

    @staticmethod
    def join(tks, sep=''):
        return sep.join(tk.content for tk in tks)

    @staticmethod
    def get_lines(tks, line_prefix=''):
        lines = []
        cur_line_parts = []

        def insert_prefix(line):
            return line_prefix + line

        for tk in tks:
            if tk.type == Token.NL:
                cur_line = insert_prefix(''.join(cur_line_parts))
                lines.append(cur_line)
                cur_line_parts = []
            else:
                cur_line_parts.append(tk.content)

        if len(cur_line_parts):
            cur_line = insert_prefix(''.join(cur_line_parts))
            lines.append(cur_line)

        return lines



class Tokenizer(object):
    '''
    Simple tokenizer of Matlab code

    Splits a string up in tokens representing a name, operator, comment,
    newline or comment.

    Current limitations:
    - no support for line continuations
    - floating point numbers may be represented by two name and one operator
    - multi-line comments are not recognized

    '''
    OP_CHARS = ['(', ')', '{', '}', '@', '^', '&', '*', '-', '+'
        , '=', ';', ':', '|', '<', '>', ',', ',/', '[', '\]', '.']

    @staticmethod
    def tokenize(string):
        lines = string.split('\n')

        tokens = []

        ws_sp = re.compile(r'(\s+)')

        op_re = ('([%s])' % ''.join(Tokenizer.OP_CHARS))
        op_sp = re.compile(op_re)

        for line in lines:
            inside_str = False
            comment_start = None
            for i, c in enumerate(line):
                if c == "'":
                    inside_str = not inside_str
                elif not inside_str and c == '%':
                    comment_start = i

            if comment_start is None:
                code_line = line
            else:
                code_line = line[:comment_start]

            for i, s in enumerate(ws_sp.split(code_line)):
                is_whitespace = i % 2 == 1
                if is_whitespace:
                    tk = Token(s, Token.WHITESPACE)
                    tokens.append(tk)
                else:
                    for j, t in enumerate(op_sp.split(s)):
                        is_op = j % 2 == 1
                        if len(t) == 0:
                            continue
                        type = Token.OP if is_op else Token.NAME
                        tk = Token(t, type)
                        tokens.append(tk)

            if comment_start is not None:
                comment = line[comment_start:]
                tk = Token(comment, Token.COMMENT)
                tokens.append(tk)

            tokens.append(Token('\n', Token.NL))

        return tokens

    @staticmethod
    def from_file(fn):
        with open(fn) as f:
            string = f.read()
            return Tokenizer.tokenize(string)



class TokenPattern(object):
    def __init__(self, content, type):
        self.content = content
        self.type = type

    def _matches(self, tk):
        raise NotImplementedError

    def __str__(self):
        return '%s(%s,%s)' % (self.__class__.__name__,
                              self.content,
                              self.type)

    @staticmethod
    def find(token_pats, tks, start=0, stop=None):
        n_pat = len(token_pats)
        n_tk = len(tks)

        for i_start in xrange(start, n_tk - n_pat):
            if stop is not None and i_start >= stop:
                break

            matches = True
            tk_pos = i_start

            for j, token_pat in enumerate(token_pats):
                tk_pos = token_pat._matches(tks, tk_pos)
                if tk_pos is None:
                    matches = False
                    break

            if matches:
                return (i_start, tk_pos)

        return None



class LiteralTokenPattern(TokenPattern):
    def _matches(self, tks, pos):
        tk = tks[pos]
        if (self.content is not None and
                    self.content != tk.content):
            return None

        if (self.type is not None and
                    self.type != tk.type):
            return None

        return pos + 1



class OptionalTypeTokenPattern(TokenPattern):
    def __init__(self, type):
        super(OptionalTypeTokenPattern, self).__init__(None, type)

    def _matches(self, tks, pos):
        for i in xrange(pos, len(tks)):
            tk = tks[i]
            same_type = self.type == tk.type

            if not same_type:
                break

        return i



class SkipUntilTokenPattern(TokenPattern):
    def _matches(self, tks, pos):
        n_tks = len(tks)
        for i in xrange(pos, n_tks):
            tk = tks[i]

            matches_content = self.content is None or tk.content == self.content
            matches_type = self.type == tk.type

            if matches_content and matches_type:
                return i

        return None



class SkipUntilTypeTokenPattern(SkipUntilTokenPattern):
    def __init__(self, type):
        super(SkipUntilTypeTokenPattern, self).__init__(None, type)




import os

import cmake_build_extension
import setuptools

if "CIBUILDWHEEL" in os.environ and os.environ["CIBUILDWHEEL"] == "1":
    CIBW_CMAKE_OPTIONS = ["-DCMAKE_INSTALL_LIBDIR=lib"]
else:
    CIBW_CMAKE_OPTIONS = []

setuptools.setup(
    cmdclass=dict(build_ext=cmake_build_extension.BuildExtension),
    ext_modules=[
        cmake_build_extension.CMakeExtension(
            name="BuildAndInstall",
            install_prefix="idyntree",
            expose_binaries=["bin/idyntree-model-info"],
            cmake_configure_options=[
                "-DBUILD_SHARED_LIBS:BOOL=OFF",
                "-DIDYNTREE_USES_PYTHON:BOOL=ON",
                "-DBUILD_TESTING:BOOL=OFF",
                "-DIDYNTREE_PACKAGE_FOR_PYPI:BOOL=ON",
                "-DIDYNTREE_USES_IPOPT:BOOL=OFF",
                "-DIDYNTREE_USES_ASSIMP:BOOL=OFF",
                "-DIDYNTREE_USES_IRRLICHT:BOOL=OFF",
                "-DIDYNTREE_USES_OSQPEIGEN:BOOL=OFF",
                "-DIDYNTREE_USES_ALGLIB:BOOL=OFF",
                "-DIDYNTREE_USES_WORHP:BOOL=OFF",
                "-DIDYNTREE_USES_YARP:BOOL=OFF",
                "-DIDYNTREE_USES_ICUB_MAIN:BOOL=OFF",
                "-DIDYNTREE_PYTHON_PIP_METADATA_INSTALL:BOOL=OFF",
            ]
            + CIBW_CMAKE_OPTIONS,
        ),
    ],
)



#!/bin/python3

# SPDX-FileCopyrightText: Fondazione Istituto Italiano di Tecnologia (IIT)
# SPDX-License-Identifier: BSD-3-Clause

import toml
import subprocess
import os
import argparse

from generate_documentation_files import generate_documentation_files

def generate_documentation(tag, options):
    print("###### Generating documentation for", tag, " ######")

    root_folder = "../"
    if tag != 'master':
        os.chdir('idyntree')
        subprocess.Popen(["git", "checkout", tag], stdout=subprocess.PIPE)
        os.chdir('..')
        root_folder = "idyntree/"

    additional_pages = []
    if 'additional_pages' in options.keys():
        additional_pages = [root_folder + page for page in options['additional_pages']]

    generate_documentation_files(tag=tag,
                                 src_folder=root_folder + options['src_folder'],
                                 main_page=root_folder + options['main_page'],
                                 additional_pages=additional_pages,
                                 input_files_path="./",
                                 output_files_path="./")

    try:
        subprocess.check_call(["python3", args.mcss_path, "conf-" + tag + ".py"])
    except subprocess.CalledProcessError as error:
        raise ValueError(error)


if __name__ == "__main__":
    parser = argparse.ArgumentParser(description='generate_website.py is a python script useful to generate the '
                                                 'documentation of the bipedal-locomotion-framework repo.')
    parser.add_argument('--mcss_path',
                        type=str,
                        required=True,
                        help='Path to \'m.css/documentation/doxygen.py\' file')
    args = parser.parse_args()

    parameters = toml.load("config.toml")

    # create the site directory structure
    os.makedirs('./site', exist_ok=True)
    for key, value in parameters.items():
        if key != 'master':
            directory = './site/' + key
            os.makedirs(directory, exist_ok=True)

    # build the documentation
    for key, value in parameters.items():
        generate_documentation(key, value)

